from __future__ import annotations

from http import HTTPStatus
from json import dumps
from typing import Any
from unittest import mock

import pytest
import requests

from hcloud import (
    APIException,
    Client,
    constant_backoff_function,
    exponential_backoff_function,
)
from hcloud._client import ClientBase, _build_user_agent


def test_exponential_backoff_function():
    backoff = exponential_backoff_function(
        base=1.0,
        multiplier=2,
        cap=60.0,
    )
    max_retries = 5

    results = [backoff(i) for i in range(max_retries)]
    assert sum(results) == 31.0
    assert results == [1.0, 2.0, 4.0, 8.0, 16.0]


def test_constant_backoff_function():
    backoff = constant_backoff_function(interval=1.0)
    max_retries = 5

    for i in range(max_retries):
        assert backoff(i) == 1.0


def test_build_user_agent():
    assert _build_user_agent(None, None) == "hcloud-python/0.0.0"
    assert _build_user_agent("my-app", None) == "my-app hcloud-python/0.0.0"
    assert _build_user_agent("my-app", "1.0.0") == "my-app/1.0.0 hcloud-python/0.0.0"
    assert _build_user_agent(None, "1.0.0") == "hcloud-python/0.0.0"


class TestClient:
    @pytest.fixture()
    def client(self):
        return Client(token="TOKEN")

    def test_request(self, client: Client):
        client._client.request = mock.MagicMock()
        client.request(method="GET", url="/path")
        client._client.request.assert_called_once_with("GET", "/path")


def make_response(
    status: HTTPStatus,
    *,
    json: Any | None = None,
    text: str | None = None,
) -> requests.Response:
    response = requests.Response()
    response.status_code = status.value
    response.reason = status.phrase

    if json is not None:
        response.headers["Content-type"] = "application/json"
        response._content = dumps(json).encode("utf-8")
    elif text is not None:
        response.headers["Content-type"] = "text/plain"
        response._content = text.encode("utf-8")
    return response


class TestBaseClient:
    @pytest.fixture()
    def client(self):
        client = ClientBase(
            token="TOKEN",
            endpoint="https://api.hetzner.cloud/v1",
        )
        client._session = mock.MagicMock()
        return client

    def test_init(self, client: ClientBase):
        assert client._user_agent == "hcloud-python/0.0.0"
        assert client._headers == {
            "User-Agent": "hcloud-python/0.0.0",
            "Authorization": "Bearer TOKEN",
            "Accept": "application/json",
        }
        assert client._poll_interval_func(1) == 1.0
        assert client._retry_interval_func(1) == pytest.approx(1.5, rel=0.5)  # Jitter

    @pytest.mark.parametrize(
        ("exception", "expected"),
        [
            (
                APIException(code="rate_limit_exceeded", message="Error", details=None),
                True,
            ),
            (
                APIException(code="conflict", message="Error", details=None),
                True,
            ),
            (
                APIException(code=409, message="Conflict", details=None),
                False,
            ),
            (
                APIException(code=429, message="Too Many Requests", details=None),
                False,
            ),
            (
                APIException(code=502, message="Bad Gateway", details=None),
                True,
            ),
            (
                APIException(code=503, message="Service Unavailable", details=None),
                False,
            ),
            (
                APIException(code=504, message="Gateway Timeout", details=None),
                True,
            ),
        ],
    )
    def test_retry_policy(
        self,
        client: ClientBase,
        exception: APIException,
        expected: bool,
    ):
        assert client._retry_policy(exception) == expected

    def test_request_200(self, client: ClientBase):
        client._session.request.return_value = make_response(
            status=HTTPStatus.OK,
            json={"result": "data"},
        )

        result = client.request(
            method="POST",
            url="/path",
            params={"argument": "value"},
            timeout=2,
        )

        client._session.request.assert_called_once_with(
            method="POST",
            url="https://api.hetzner.cloud/v1/path",
            headers={
                "User-Agent": "hcloud-python/0.0.0",
                "Authorization": "Bearer TOKEN",
                "Accept": "application/json",
            },
            params={"argument": "value"},
            timeout=2,
        )
        assert result == {"result": "data"}

    def test_request_200_empty_content(self, client: ClientBase):
        client._session.request.return_value = make_response(
            status=HTTPStatus.OK,
            text="",
        )

        result = client.request(method="POST", url="/path")
        assert result == {}

    def test_request_fail_200_invalid_json(self, client: ClientBase):
        client._session.request.return_value = make_response(
            status=HTTPStatus.OK,
            text="{'key': 'value'",
        )

        with pytest.raises(APIException) as exc:
            client.request(method="POST", url="/path")

        assert exc.value.code == 200
        assert exc.value.message == "OK"
        assert exc.value.details["content"] == b"{'key': 'value'"

    def test_request_fail_422(self, client: ClientBase):
        client._session.request.return_value = make_response(
            status=HTTPStatus.UNPROCESSABLE_ENTITY,
            json={
                "error": {
                    "code": "invalid_input",
                    "message": "invalid input in field 'broken_field': is too long",
                    "details": {
                        "fields": [
                            {"name": "broken_field", "messages": ["is too long"]}
                        ]
                    },
                }
            },
        )

        with pytest.raises(APIException) as exc:
            client.request(method="POST", url="/path")

        assert exc.value.code == "invalid_input"
        assert exc.value.message == "invalid input in field 'broken_field': is too long"
        assert exc.value.details["fields"][0]["name"] == "broken_field"

    def test_request_fail_422_correlation_id(self, client: ClientBase):
        response = make_response(
            status=HTTPStatus.UNPROCESSABLE_ENTITY,
            json={
                "error": {
                    "code": "service_error",
                    "message": "Something crashed",
                }
            },
        )
        response.headers["X-Correlation-Id"] = "67ed842dc8bc8673"
        client._session.request.return_value = response

        with pytest.raises(APIException) as exc:
            client.request(method="POST", url="/path")

        assert exc.value.code == "service_error"
        assert exc.value.message == "Something crashed"
        assert exc.value.details is None
        assert exc.value.correlation_id == "67ed842dc8bc8673"
        assert str(exc.value) == "Something crashed (service_error, 67ed842dc8bc8673)"

    def test_request_fail_500(self, client: ClientBase):
        client._session.request.return_value = make_response(
            status=HTTPStatus.INTERNAL_SERVER_ERROR,
            text="Internal Server Error",
        )

        with pytest.raises(APIException) as exc:
            client.request(method="POST", url="/path")

        assert exc.value.code == 500
        assert exc.value.message == "Internal Server Error"
        assert exc.value.details["content"] == b"Internal Server Error"

    def test_request_fail_500_no_content(self, client: ClientBase):
        client._session.request.return_value = make_response(
            status=HTTPStatus.INTERNAL_SERVER_ERROR,
        )

        with pytest.raises(APIException) as exc:
            client.request(method="POST", url="/path")

        assert exc.value.code == 500
        assert exc.value.message == "Internal Server Error"
        assert exc.value.details["content"] is None
        assert str(exc.value) == "Internal Server Error (500)"

    def test_request_fail_419(self, client: ClientBase):
        client._retry_interval_func = constant_backoff_function(0.0)

        client._session.request.return_value = make_response(
            status=HTTPStatus.TOO_MANY_REQUESTS,
            json={
                "error": {
                    "code": "rate_limit_exceeded",
                    "message": "limit of 3600 requests per hour reached",
                    "details": None,
                }
            },
        )
        with pytest.raises(APIException) as exc:
            client.request(method="POST", url="/path")

        assert client._session.request.call_count == 6
        assert exc.value.code == "rate_limit_exceeded"
        assert exc.value.message == "limit of 3600 requests per hour reached"

    def test_request_fail_419_recover(self, client: ClientBase):
        client._retry_interval_func = constant_backoff_function(0.0)

        client._session.request.side_effect = [
            make_response(
                status=HTTPStatus.TOO_MANY_REQUESTS,
                json={
                    "error": {
                        "code": "rate_limit_exceeded",
                        "message": "limit of 3600 requests per hour reached",
                        "details": None,
                    }
                },
            ),
            make_response(
                status=HTTPStatus.OK,
                json={"result": "data"},
            ),
        ]

        result = client.request(method="GET", url="/path")

        assert client._session.request.call_count == 2
        assert result == {"result": "data"}

    def test_request_fail_timeout(self, client: ClientBase):
        client._retry_interval_func = constant_backoff_function(0.0)
        client._session.request.side_effect = requests.exceptions.Timeout("timeout")

        with pytest.raises(requests.exceptions.Timeout) as exc:
            client.request(method="GET", url="/path")

        assert str(exc.value) == "timeout"
        assert client._session.request.call_count == 6

    def test_request_fail_timeout_recover(self, client: ClientBase):
        client._retry_interval_func = constant_backoff_function(0.0)

        client._session.request.side_effect = [
            requests.exceptions.Timeout("timeout"),
            make_response(
                status=HTTPStatus.OK,
                json={"result": "data"},
            ),
        ]

        result = client.request(method="GET", url="/path")

        assert client._session.request.call_count == 2
        assert result == {"result": "data"}



from __future__ import annotations

import pytest

from hcloud.storage_box_types import StorageBoxType


@pytest.mark.parametrize(
    "value",
    [
        (StorageBoxType(id=1),),
    ],
)
def test_eq(value):
    assert value.__eq__(value)



from __future__ import annotations

from unittest import mock

import pytest

from hcloud import Client
from hcloud.floating_ips import BoundFloatingIP, FloatingIP, FloatingIPsClient
from hcloud.locations import BoundLocation, Location
from hcloud.servers import BoundServer, Server

from ..conftest import BoundModelTestCase


class TestBoundFloatingIP(BoundModelTestCase):
    methods = [
        BoundFloatingIP.update,
        BoundFloatingIP.delete,
        BoundFloatingIP.change_protection,
        BoundFloatingIP.change_dns_ptr,
        BoundFloatingIP.assign,
        BoundFloatingIP.unassign,
    ]

    @pytest.fixture()
    def resource_client(self, client: Client):
        return client.floating_ips

    @pytest.fixture()
    def bound_model(self, resource_client, floating_ip_response):
        return BoundFloatingIP(
            resource_client, data=floating_ip_response["floating_ip"]
        )

    def test_init(self, bound_model: BoundFloatingIP):
        o = bound_model
        assert o.id == 4711
        assert o.description == "Web Frontend"
        assert o.name == "Web Frontend"
        assert o.ip == "131.232.99.1"
        assert o.type == "ipv4"
        assert o.protection == {"delete": False}
        assert o.labels == {}
        assert o.blocked is False

        assert isinstance(o.server, BoundServer)
        assert o.server.id == 42

        assert isinstance(o.home_location, BoundLocation)
        assert o.home_location.id == 1
        assert o.home_location.name == "fsn1"
        assert o.home_location.description == "Falkenstein DC Park 1"
        assert o.home_location.country == "DE"
        assert o.home_location.city == "Falkenstein"
        assert o.home_location.latitude == 50.47612
        assert o.home_location.longitude == 12.370071


class TestFloatingIPsClient:
    @pytest.fixture()
    def floating_ips_client(self, client: Client):
        return FloatingIPsClient(client)

    def test_get_by_id(
        self,
        request_mock: mock.MagicMock,
        floating_ips_client: FloatingIPsClient,
        floating_ip_response,
    ):
        request_mock.return_value = floating_ip_response

        bound_floating_ip = floating_ips_client.get_by_id(1)

        request_mock.assert_called_with(
            method="GET",
            url="/floating_ips/1",
        )
        assert bound_floating_ip._client is floating_ips_client
        assert bound_floating_ip.id == 4711
        assert bound_floating_ip.description == "Web Frontend"

    def test_get_by_name(
        self,
        request_mock: mock.MagicMock,
        floating_ips_client: FloatingIPsClient,
        one_floating_ips_response,
    ):
        request_mock.return_value = one_floating_ips_response

        bound_floating_ip = floating_ips_client.get_by_name("Web Frontend")

        request_mock.assert_called_with(
            method="GET",
            url="/floating_ips",
            params={"name": "Web Frontend"},
        )
        assert bound_floating_ip._client is floating_ips_client
        assert bound_floating_ip.id == 4711
        assert bound_floating_ip.name == "Web Frontend"
        assert bound_floating_ip.description == "Web Frontend"

    @pytest.mark.parametrize(
        "params",
        [{"label_selector": "label1", "page": 1, "per_page": 10}, {"name": ""}, {}],
    )
    def test_get_list(
        self,
        request_mock: mock.MagicMock,
        floating_ips_client: FloatingIPsClient,
        two_floating_ips_response,
        params,
    ):
        request_mock.return_value = two_floating_ips_response

        result = floating_ips_client.get_list(**params)

        request_mock.assert_called_with(
            method="GET",
            url="/floating_ips",
            params=params,
        )

        bound_floating_ips = result.floating_ips
        assert result.meta is not None

        assert len(bound_floating_ips) == 2

        bound_floating_ip1 = bound_floating_ips[0]
        bound_floating_ip2 = bound_floating_ips[1]

        assert bound_floating_ip1._client is floating_ips_client
        assert bound_floating_ip1.id == 4711
        assert bound_floating_ip1.description == "Web Frontend"

        assert bound_floating_ip2._client is floating_ips_client
        assert bound_floating_ip2.id == 4712
        assert bound_floating_ip2.description == "Web Backend"

    @pytest.mark.parametrize("params", [{"label_selector": "label1"}, {}])
    def test_get_all(
        self,
        request_mock: mock.MagicMock,
        floating_ips_client: FloatingIPsClient,
        two_floating_ips_response,
        params,
    ):
        request_mock.return_value = two_floating_ips_response

        bound_floating_ips = floating_ips_client.get_all(**params)

        params.update({"page": 1, "per_page": 50})

        request_mock.assert_called_with(
            method="GET",
            url="/floating_ips",
            params=params,
        )

        assert len(bound_floating_ips) == 2

        bound_floating_ip1 = bound_floating_ips[0]
        bound_floating_ip2 = bound_floating_ips[1]

        assert bound_floating_ip1._client is floating_ips_client
        assert bound_floating_ip1.id == 4711
        assert bound_floating_ip1.description == "Web Frontend"

        assert bound_floating_ip2._client is floating_ips_client
        assert bound_floating_ip2.id == 4712
        assert bound_floating_ip2.description == "Web Backend"

    def test_create_with_location(
        self,
        request_mock: mock.MagicMock,
        floating_ips_client: FloatingIPsClient,
        floating_ip_response,
    ):
        request_mock.return_value = floating_ip_response

        response = floating_ips_client.create(
            "ipv6", "Web Frontend", home_location=Location(name="location")
        )

        request_mock.assert_called_with(
            method="POST",
            url="/floating_ips",
            json={
                "description": "Web Frontend",
                "type": "ipv6",
                "home_location": "location",
            },
        )

        bound_floating_ip = response.floating_ip
        action = response.action

        assert bound_floating_ip._client is floating_ips_client
        assert bound_floating_ip.id == 4711
        assert bound_floating_ip.description == "Web Frontend"
        assert action is None

    @pytest.mark.parametrize(
        "server", [Server(id=1), BoundServer(mock.MagicMock(), dict(id=1))]
    )
    def test_create_with_server(
        self,
        request_mock: mock.MagicMock,
        floating_ips_client: FloatingIPsClient,
        server,
        floating_ip_create_response,
    ):
        request_mock.return_value = floating_ip_create_response

        response = floating_ips_client.create(
            type="ipv6", description="Web Frontend", server=server
        )

        request_mock.assert_called_with(
            method="POST",
            url="/floating_ips",
            json={"description": "Web Frontend", "type": "ipv6", "server": 1},
        )
        bound_floating_ip = response.floating_ip
        action = response.action

        assert bound_floating_ip._client is floating_ips_client
        assert bound_floating_ip.id == 4711
        assert bound_floating_ip.description == "Web Frontend"
        assert action.id == 13

    def test_create_with_name(
        self,
        request_mock: mock.MagicMock,
        floating_ips_client: FloatingIPsClient,
        floating_ip_create_response,
    ):
        request_mock.return_value = floating_ip_create_response

        response = floating_ips_client.create(
            type="ipv6", description="Web Frontend", name="Web Frontend"
        )

        request_mock.assert_called_with(
            method="POST",
            url="/floating_ips",
            json={
                "description": "Web Frontend",
                "type": "ipv6",
                "name": "Web Frontend",
            },
        )
        bound_floating_ip = response.floating_ip
        action = response.action

        assert bound_floating_ip._client is floating_ips_client
        assert bound_floating_ip.id == 4711
        assert bound_floating_ip.description == "Web Frontend"
        assert bound_floating_ip.name == "Web Frontend"
        assert action.id == 13

    @pytest.mark.parametrize(
        "floating_ip", [FloatingIP(id=1), BoundFloatingIP(mock.MagicMock(), dict(id=1))]
    )
    def test_update(
        self,
        request_mock: mock.MagicMock,
        floating_ips_client: FloatingIPsClient,
        floating_ip,
        response_update_floating_ip,
    ):
        request_mock.return_value = response_update_floating_ip

        floating_ip = floating_ips_client.update(
            floating_ip, description="New description", name="New name"
        )

        request_mock.assert_called_with(
            method="PUT",
            url="/floating_ips/1",
            json={"description": "New description", "name": "New name"},
        )

        assert floating_ip.id == 4711
        assert floating_ip.description == "New description"
        assert floating_ip.name == "New name"

    @pytest.mark.parametrize(
        "floating_ip", [FloatingIP(id=1), BoundFloatingIP(mock.MagicMock(), dict(id=1))]
    )
    def test_change_protection(
        self,
        request_mock: mock.MagicMock,
        floating_ips_client: FloatingIPsClient,
        floating_ip,
        action_response,
    ):
        request_mock.return_value = action_response

        action = floating_ips_client.change_protection(floating_ip, True)

        request_mock.assert_called_with(
            method="POST",
            url="/floating_ips/1/actions/change_protection",
            json={"delete": True},
        )

        assert action.id == 1
        assert action.progress == 0

    @pytest.mark.parametrize(
        "floating_ip", [FloatingIP(id=1), BoundFloatingIP(mock.MagicMock(), dict(id=1))]
    )
    def test_delete(
        self,
        request_mock: mock.MagicMock,
        floating_ips_client: FloatingIPsClient,
        floating_ip,
        action_response,
    ):
        request_mock.return_value = action_response

        delete_success = floating_ips_client.delete(floating_ip)

        request_mock.assert_called_with(
            method="DELETE",
            url="/floating_ips/1",
        )

        assert delete_success is True

    @pytest.mark.parametrize(
        "server,floating_ip",
        [
            (Server(id=1), FloatingIP(id=12)),
            (
                BoundServer(mock.MagicMock(), dict(id=1)),
                BoundFloatingIP(mock.MagicMock(), dict(id=12)),
            ),
        ],
    )
    def test_assign(
        self,
        request_mock: mock.MagicMock,
        floating_ips_client: FloatingIPsClient,
        server,
        floating_ip,
        action_response,
    ):
        request_mock.return_value = action_response

        action = floating_ips_client.assign(floating_ip, server)

        request_mock.assert_called_with(
            method="POST",
            url="/floating_ips/12/actions/assign",
            json={"server": 1},
        )
        assert action.id == 1
        assert action.progress == 0

    @pytest.mark.parametrize(
        "floating_ip",
        [FloatingIP(id=12), BoundFloatingIP(mock.MagicMock(), dict(id=12))],
    )
    def test_unassign(
        self,
        request_mock: mock.MagicMock,
        floating_ips_client: FloatingIPsClient,
        floating_ip,
        action_response,
    ):
        request_mock.return_value = action_response

        action = floating_ips_client.unassign(floating_ip)

        request_mock.assert_called_with(
            method="POST",
            url="/floating_ips/12/actions/unassign",
        )
        assert action.id == 1
        assert action.progress == 0

    @pytest.mark.parametrize(
        "floating_ip",
        [FloatingIP(id=12), BoundFloatingIP(mock.MagicMock(), dict(id=12))],
    )
    def test_change_dns_ptr(
        self,
        request_mock: mock.MagicMock,
        floating_ips_client: FloatingIPsClient,
        floating_ip,
        action_response,
    ):
        request_mock.return_value = action_response

        action = floating_ips_client.change_dns_ptr(
            floating_ip, "1.2.3.4", "server02.example.com"
        )

        request_mock.assert_called_with(
            method="POST",
            url="/floating_ips/12/actions/change_dns_ptr",
            json={"ip": "1.2.3.4", "dns_ptr": "server02.example.com"},
        )
        assert action.id == 1
        assert action.progress == 0



# pylint: disable=protected-access

from __future__ import annotations

from unittest import mock

import pytest
from dateutil.parser import isoparse

from hcloud import Client
from hcloud.zones import (
    BoundZone,
    BoundZoneRRSet,
    Zone,
    ZoneAuthoritativeNameservers,
    ZonePrimaryNameserver,
    ZoneRecord,
    ZoneRRSet,
    ZonesClient,
)

from ..conftest import BoundModelTestCase, assert_bound_action1


def assert_bound_zone1(o: BoundZone, client: ZonesClient):
    assert isinstance(o, BoundZone)
    assert o._client is client
    assert o.id == 42
    assert o.name == "example1.com"


def assert_bound_zone2(o: BoundZone, client: ZonesClient):
    assert isinstance(o, BoundZone)
    assert o._client is client
    assert o.id == 43
    assert o.name == "example2.com"


def assert_bound_zone_rrset1(o: BoundZoneRRSet, client: ZonesClient):
    assert isinstance(o, BoundZoneRRSet)
    assert o._client is client
    assert o.name == "www"
    assert o.type == "A"
    assert o.id == "www/A"


def assert_bound_zone_rrset2(o: BoundZoneRRSet, client: ZonesClient):
    assert isinstance(o, BoundZoneRRSet)
    assert o._client is client
    assert o.name == "blog"
    assert o.type == "A"
    assert o.id == "blog/A"


class TestZonesClient:
    @pytest.fixture()
    def resource_client(self, client: Client) -> ZonesClient:
        return client.zones

    def test_get_using_id(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone_response,
    ):
        request_mock.return_value = zone_response

        result = resource_client.get(42)

        request_mock.assert_called_with(
            method="GET",
            url="/zones/42",
        )

        assert_bound_zone1(result, resource_client)
        assert result.created == isoparse("2016-01-30T23:55:00+00:00")
        assert result.mode == "primary"
        assert result.ttl == 10800
        assert result.protection == {"delete": False}
        assert result.labels == {"key": "value"}
        assert result.primary_nameservers[0].address == "198.51.100.1"
        assert result.primary_nameservers[0].port == 53
        assert result.primary_nameservers[1].address == "203.0.113.1"
        assert result.primary_nameservers[1].port == 53

        assert (
            result.authoritative_nameservers.assigned[0] == "hydrogen.ns.hetzner.com."
        )
        assert (
            result.authoritative_nameservers.delegated[0] == "hydrogen.ns.hetzner.com."
        )
        assert result.authoritative_nameservers.delegation_last_check == isoparse(
            "2016-01-30T23:55:00+00:00"
        )
        assert result.authoritative_nameservers.delegation_status == "valid"

        assert result.record_count == 0
        assert result.status == "ok"
        assert result.registrar == "hetzner"

    def test_get_using_name(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone_response,
    ):
        request_mock.return_value = zone_response

        result = resource_client.get("example.com")

        request_mock.assert_called_with(
            method="GET",
            url="/zones/example.com",
        )

        assert_bound_zone1(result, resource_client)

    @pytest.mark.parametrize(
        "params",
        [
            {"mode": "primary"},
            {"label_selector": "key=value", "page": 2, "per_page": 10, "sort": "id"},
            {"name": "example.com"},
            {},
        ],
    )
    def test_get_list(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone_list_response,
        params,
    ):
        request_mock.return_value = zone_list_response

        resp = resource_client.get_list(**params)

        request_mock.assert_called_with(
            method="GET",
            url="/zones",
            params=params,
        )

        assert resp.meta is not None

        assert len(resp.zones) == 2
        assert_bound_zone1(resp.zones[0], resource_client)
        assert_bound_zone2(resp.zones[1], resource_client)

    @pytest.mark.parametrize(
        "params",
        [
            {"label_selector": "key=value"},
            {},
        ],
    )
    def test_get_all(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone_list_response,
        params,
    ):
        request_mock.return_value = zone_list_response

        result = resource_client.get_all(**params)

        request_mock.assert_called_with(
            method="GET",
            url="/zones",
            params={**params, "page": 1, "per_page": 50},
        )

        assert len(result) == 2
        assert_bound_zone1(result[0], resource_client)
        assert_bound_zone2(result[1], resource_client)

    def test_create(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone_create_response,
    ):
        request_mock.return_value = zone_create_response

        resp = resource_client.create(
            name="example.com",
            mode="primary",
            ttl=3600,
            labels={"key": "value"},
            primary_nameservers=[
                ZonePrimaryNameserver(address="198.51.100.1", port=53),
                ZonePrimaryNameserver(address="203.0.113.1"),
            ],
            rrsets=[ZoneRRSet(name="www", type="A", records=[ZoneRecord("127.0.0.1")])],
        )

        request_mock.assert_called_with(
            url="/zones",
            method="POST",
            json={
                "name": "example.com",
                "mode": "primary",
                "ttl": 3600,
                "labels": {"key": "value"},
                "primary_nameservers": [
                    {"address": "198.51.100.1", "port": 53},
                    {"address": "203.0.113.1"},
                ],
                "rrsets": [
                    {"name": "www", "type": "A", "records": [{"value": "127.0.0.1"}]}
                ],
            },
        )

        assert_bound_zone1(resp.zone, resource_client)
        assert_bound_action1(resp.action, resource_client._parent.actions)

    @pytest.mark.parametrize(
        "zone",
        [
            Zone(name="example.com"),
            BoundZone(client=mock.MagicMock(), data={"id": 42}),
        ],
    )
    def test_update(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone: Zone,
        zone_response,
    ):
        request_mock.return_value = zone_response

        result = resource_client.update(zone, labels={"key": "new value"})

        request_mock.assert_called_with(
            method="PUT",
            url=f"/zones/{zone.id_or_name}",
            json={"labels": {"key": "new value"}},
        )

        assert_bound_zone1(result, resource_client)

    @pytest.mark.parametrize(
        "zone",
        [
            Zone(name="example.com"),
            BoundZone(client=mock.MagicMock(), data={"id": 42}),
        ],
    )
    def test_delete(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone: Zone,
        action_response,
    ):
        request_mock.return_value = action_response

        resp = resource_client.delete(zone)

        request_mock.assert_called_with(
            method="DELETE",
            url=f"/zones/{zone.id_or_name}",
        )

        assert_bound_action1(resp.action, resource_client._parent.actions)

    @pytest.mark.parametrize(
        "zone",
        [
            Zone(name="example.com"),
            BoundZone(client=mock.MagicMock(), data={"id": 42}),
        ],
    )
    def test_export_zonefile(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone: Zone,
    ):
        request_mock.return_value = {"zonefile": "content"}

        resp = resource_client.export_zonefile(zone)

        request_mock.assert_called_with(
            method="GET",
            url=f"/zones/{zone.id_or_name}/zonefile",
        )

        assert resp.zonefile == "content"

    @pytest.mark.parametrize(
        "zone",
        [
            Zone(name="example.com"),
            BoundZone(client=mock.MagicMock(), data={"id": 42}),
        ],
    )
    def test_import_zonefile(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone: Zone,
        action_response,
    ):
        request_mock.return_value = action_response

        action = resource_client.import_zonefile(zone, "content")

        request_mock.assert_called_with(
            method="POST",
            url=f"/zones/{zone.id_or_name}/actions/import_zonefile",
            json={"zonefile": "content"},
        )

        assert_bound_action1(action, resource_client._parent.actions)

    @pytest.mark.parametrize(
        "zone",
        [
            Zone(name="example.com"),
            BoundZone(client=mock.MagicMock(), data={"id": 42}),
        ],
    )
    def test_change_protection(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone: Zone,
        action_response,
    ):
        request_mock.return_value = action_response

        action = resource_client.change_protection(zone, delete=True)

        request_mock.assert_called_with(
            method="POST",
            url=f"/zones/{zone.id_or_name}/actions/change_protection",
            json={"delete": True},
        )

        assert_bound_action1(action, resource_client._parent.actions)

    @pytest.mark.parametrize(
        "zone",
        [
            Zone(name="example.com"),
            BoundZone(client=mock.MagicMock(), data={"id": 42}),
        ],
    )
    def test_change_primary_nameservers(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone: Zone,
        action_response,
    ):
        request_mock.return_value = action_response

        action = resource_client.change_primary_nameservers(
            zone,
            primary_nameservers=[
                ZonePrimaryNameserver(address="198.51.100.1", port=53),
                ZonePrimaryNameserver(address="203.0.113.1"),
            ],
        )

        request_mock.assert_called_with(
            method="POST",
            url=f"/zones/{zone.id_or_name}/actions/change_primary_nameservers",
            json={
                "primary_nameservers": [
                    {"address": "198.51.100.1", "port": 53},
                    {"address": "203.0.113.1"},
                ]
            },
        )

        assert_bound_action1(action, resource_client._parent.actions)

    @pytest.mark.parametrize(
        "zone",
        [
            Zone(name="example.com"),
            BoundZone(client=mock.MagicMock(), data={"id": 42}),
        ],
    )
    def test_change_ttl(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone: Zone,
        action_response,
    ):
        request_mock.return_value = action_response

        action = resource_client.change_ttl(zone, 3600)

        request_mock.assert_called_with(
            method="POST",
            url=f"/zones/{zone.id_or_name}/actions/change_ttl",
            json={"ttl": 3600},
        )

        assert_bound_action1(action, resource_client._parent.actions)

    # ============ RRSETS ============

    @pytest.mark.parametrize(
        "zone",
        [
            Zone(name="example.com"),
            BoundZone(client=mock.MagicMock(), data={"id": 42}),
        ],
    )
    def test_get_rrset(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone: Zone,
        zone_rrset_response,
    ):
        request_mock.return_value = zone_rrset_response

        result = resource_client.get_rrset(zone, "www", "A")

        request_mock.assert_called_with(
            method="GET",
            url=f"/zones/{zone.id_or_name}/rrsets/www/A",
        )

        assert_bound_zone_rrset1(result, resource_client)
        assert result.ttl == 3600
        assert result.protection == {"change": False}
        assert result.labels == {"key": "value"}
        assert result.records[0].value == "198.51.100.1"
        assert result.records[0].comment == "web server"

    @pytest.mark.parametrize(
        "zone",
        [
            Zone(name="example.com"),
            BoundZone(client=mock.MagicMock(), data={"id": 42}),
        ],
    )
    @pytest.mark.parametrize(
        "params",
        [
            {"type": ["A"]},
            {"label_selector": "key=value", "page": 2, "per_page": 10, "sort": "id"},
            {"name": "www"},
            {},
        ],
    )
    def test_get_rrset_list(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone: Zone,
        zone_rrset_list_response,
        params,
    ):
        request_mock.return_value = zone_rrset_list_response

        resp = resource_client.get_rrset_list(zone, **params)

        request_mock.assert_called_with(
            method="GET",
            url=f"/zones/{zone.id_or_name}/rrsets",
            params=params,
        )

        assert resp.meta is not None

        assert len(resp.rrsets) == 2
        assert_bound_zone_rrset1(resp.rrsets[0], resource_client)
        assert_bound_zone_rrset2(resp.rrsets[1], resource_client)

    @pytest.mark.parametrize(
        "zone",
        [
            Zone(name="example.com"),
            BoundZone(client=mock.MagicMock(), data={"id": 42}),
        ],
    )
    @pytest.mark.parametrize(
        "params",
        [
            {"label_selector": "key=value"},
            {},
        ],
    )
    def test_get_rrset_all(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone: Zone,
        zone_rrset_list_response,
        params,
    ):
        request_mock.return_value = zone_rrset_list_response

        result = resource_client.get_rrset_all(zone, **params)

        request_mock.assert_called_with(
            method="GET",
            url=f"/zones/{zone.id_or_name}/rrsets",
            params={**params, "page": 1, "per_page": 50},
        )

        assert len(result) == 2
        assert_bound_zone_rrset1(result[0], resource_client)
        assert_bound_zone_rrset2(result[1], resource_client)

    @pytest.mark.parametrize(
        "zone",
        [
            Zone(name="example.com"),
            BoundZone(client=mock.MagicMock(), data={"id": 42}),
        ],
    )
    def test_create_rrset(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone: Zone,
        zone_rrset_create_response,
    ):
        request_mock.return_value = zone_rrset_create_response

        resp = resource_client.create_rrset(
            zone,
            name="www",
            type="A",
            ttl=3600,
            labels={"key": "value"},
            records=[
                ZoneRecord("198.51.100.1", "web server"),
                ZoneRecord("127.0.0.1"),
            ],
        )

        request_mock.assert_called_with(
            method="POST",
            url=f"/zones/{zone.id_or_name}/rrsets",
            json={
                "name": "www",
                "type": "A",
                "ttl": 3600,
                "labels": {"key": "value"},
                "records": [
                    {"value": "198.51.100.1", "comment": "web server"},
                    {"value": "127.0.0.1"},
                ],
            },
        )

        assert_bound_zone_rrset1(resp.rrset, resource_client)
        assert_bound_action1(resp.action, resource_client._parent.actions)

    @pytest.mark.parametrize(
        "zone",
        [
            Zone(name="example.com"),
            BoundZone(client=mock.MagicMock(), data={"id": 42}),
        ],
    )
    @pytest.mark.parametrize(
        "rrset",
        [
            ZoneRRSet(name="www", type="A"),
            BoundZoneRRSet(client=mock.MagicMock(), data={"id": "www/A"}),
        ],
    )
    def test_update_rrset(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone: Zone,
        rrset: ZoneRRSet,
        zone_rrset_response,
    ):
        rrset.zone = zone

        request_mock.return_value = zone_rrset_response

        result = resource_client.update_rrset(rrset, labels={"key": "new value"})

        request_mock.assert_called_with(
            method="PUT",
            url=f"/zones/{zone.id_or_name}/rrsets/www/A",
            json={"labels": {"key": "new value"}},
        )

        assert_bound_zone_rrset1(result, resource_client)

    @pytest.mark.parametrize(
        "zone",
        [
            Zone(name="example.com"),
            BoundZone(client=mock.MagicMock(), data={"id": 42}),
        ],
    )
    @pytest.mark.parametrize(
        "rrset",
        [
            ZoneRRSet(name="www", type="A"),
            BoundZoneRRSet(client=mock.MagicMock(), data={"id": "www/A"}),
        ],
    )
    def test_delete_rrset(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone: Zone,
        rrset: ZoneRRSet,
        action_response,
    ):
        rrset.zone = zone

        request_mock.return_value = action_response

        resp = resource_client.delete_rrset(rrset)

        request_mock.assert_called_with(
            method="DELETE",
            url=f"/zones/{zone.id_or_name}/rrsets/www/A",
        )

        assert_bound_action1(resp.action, resource_client._parent.actions)

    @pytest.mark.parametrize(
        "zone",
        [
            Zone(name="example.com"),
            BoundZone(client=mock.MagicMock(), data={"id": 42}),
        ],
    )
    @pytest.mark.parametrize(
        "rrset",
        [
            ZoneRRSet(name="www", type="A"),
            BoundZoneRRSet(client=mock.MagicMock(), data={"id": "www/A"}),
        ],
    )
    def test_change_rrset_protection(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone: Zone,
        rrset: ZoneRRSet,
        action_response,
    ):
        rrset.zone = zone

        request_mock.return_value = action_response

        action = resource_client.change_rrset_protection(rrset, change=True)

        request_mock.assert_called_with(
            method="POST",
            url=f"/zones/{zone.id_or_name}/rrsets/{rrset.name}/{rrset.type}/actions/change_protection",
            json={"change": True},
        )

        assert_bound_action1(action, resource_client._parent.actions)

    @pytest.mark.parametrize(
        "zone",
        [
            Zone(name="example.com"),
            BoundZone(client=mock.MagicMock(), data={"id": 42}),
        ],
    )
    @pytest.mark.parametrize(
        "rrset",
        [
            ZoneRRSet(name="www", type="A"),
            BoundZoneRRSet(client=mock.MagicMock(), data={"id": "www/A"}),
        ],
    )
    def test_change_rrset_ttl(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone: Zone,
        rrset: ZoneRRSet,
        action_response,
    ):
        rrset.zone = zone

        request_mock.return_value = action_response

        action = resource_client.change_rrset_ttl(rrset, ttl=3600)

        request_mock.assert_called_with(
            method="POST",
            url=f"/zones/{zone.id_or_name}/rrsets/{rrset.name}/{rrset.type}/actions/change_ttl",
            json={"ttl": 3600},
        )

        assert_bound_action1(action, resource_client._parent.actions)

    @pytest.mark.parametrize(
        "zone",
        [
            Zone(name="example.com"),
            BoundZone(client=mock.MagicMock(), data={"id": 42}),
        ],
    )
    @pytest.mark.parametrize(
        "rrset",
        [
            ZoneRRSet(name="www", type="A"),
            BoundZoneRRSet(client=mock.MagicMock(), data={"id": "www/A"}),
        ],
    )
    def test_add_rrset_records(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone: Zone,
        rrset: ZoneRRSet,
        action_response,
    ):
        rrset.zone = zone

        request_mock.return_value = action_response

        action = resource_client.add_rrset_records(
            rrset,
            records=[
                ZoneRecord("198.51.100.1", "web server"),
                ZoneRecord("127.0.0.1"),
            ],
            ttl=300,
        )

        request_mock.assert_called_with(
            method="POST",
            url=f"/zones/{zone.id_or_name}/rrsets/{rrset.name}/{rrset.type}/actions/add_records",
            json={
                "records": [
                    {"value": "198.51.100.1", "comment": "web server"},
                    {"value": "127.0.0.1"},
                ],
                "ttl": 300,
            },
        )

        assert_bound_action1(action, resource_client._parent.actions)

    @pytest.mark.parametrize(
        "zone",
        [
            Zone(name="example.com"),
            BoundZone(client=mock.MagicMock(), data={"id": 42}),
        ],
    )
    @pytest.mark.parametrize(
        "rrset",
        [
            ZoneRRSet(name="www", type="A"),
            BoundZoneRRSet(client=mock.MagicMock(), data={"id": "www/A"}),
        ],
    )
    def test_update_rrset_records(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone: Zone,
        rrset: ZoneRRSet,
        action_response,
    ):
        rrset.zone = zone

        request_mock.return_value = action_response

        action = resource_client.update_rrset_records(
            rrset,
            records=[
                ZoneRecord("198.51.100.1", "web server"),
                ZoneRecord("198.51.100.2", ""),
                ZoneRecord("127.0.0.1"),
            ],
        )

        request_mock.assert_called_with(
            method="POST",
            url=f"/zones/{zone.id_or_name}/rrsets/{rrset.name}/{rrset.type}/actions/update_records",
            json={
                "records": [
                    {"value": "198.51.100.1", "comment": "web server"},
                    {"value": "198.51.100.2", "comment": ""},
                    {"value": "127.0.0.1"},
                ],
            },
        )

        assert_bound_action1(action, resource_client._parent.actions)

    @pytest.mark.parametrize(
        "zone",
        [
            Zone(name="example.com"),
            BoundZone(client=mock.MagicMock(), data={"id": 42}),
        ],
    )
    @pytest.mark.parametrize(
        "rrset",
        [
            ZoneRRSet(name="www", type="A"),
            BoundZoneRRSet(client=mock.MagicMock(), data={"id": "www/A"}),
        ],
    )
    def test_remove_rrset_records(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone: Zone,
        rrset: ZoneRRSet,
        action_response,
    ):
        rrset.zone = zone

        request_mock.return_value = action_response

        action = resource_client.remove_rrset_records(
            rrset,
            records=[
                ZoneRecord("198.51.100.1", "web server"),
                ZoneRecord("127.0.0.1"),
            ],
        )

        request_mock.assert_called_with(
            method="POST",
            url=f"/zones/{zone.id_or_name}/rrsets/{rrset.name}/{rrset.type}/actions/remove_records",
            json={
                "records": [
                    {"value": "198.51.100.1", "comment": "web server"},
                    {"value": "127.0.0.1"},
                ]
            },
        )

        assert_bound_action1(action, resource_client._parent.actions)

    @pytest.mark.parametrize(
        "zone",
        [
            Zone(name="example.com"),
            BoundZone(client=mock.MagicMock(), data={"id": 42}),
        ],
    )
    @pytest.mark.parametrize(
        "rrset",
        [
            ZoneRRSet(name="www", type="A"),
            BoundZoneRRSet(client=mock.MagicMock(), data={"id": "www/A"}),
        ],
    )
    def test_set_rrset_records(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone: Zone,
        rrset: ZoneRRSet,
        action_response,
    ):
        rrset.zone = zone

        request_mock.return_value = action_response

        action = resource_client.set_rrset_records(
            rrset,
            records=[
                ZoneRecord("198.51.100.1", "web server"),
                ZoneRecord("127.0.0.1"),
            ],
        )

        request_mock.assert_called_with(
            method="POST",
            url=f"/zones/{zone.id_or_name}/rrsets/{rrset.name}/{rrset.type}/actions/set_records",
            json={
                "records": [
                    {"value": "198.51.100.1", "comment": "web server"},
                    {"value": "127.0.0.1"},
                ]
            },
        )

        assert_bound_action1(action, resource_client._parent.actions)


class TestBoundZone(BoundModelTestCase):
    methods = [
        BoundZone.update,
        BoundZone.delete,
        BoundZone.import_zonefile,
        BoundZone.export_zonefile,
        BoundZone.change_primary_nameservers,
        BoundZone.change_ttl,
        BoundZone.change_protection,
        BoundZone.get_rrset_all,
        BoundZone.get_rrset_list,
        BoundZone.get_rrset,
        BoundZone.create_rrset,
        # With rrset sub resource
        (BoundZone.update_rrset, {"sub_resource": True}),
        (BoundZone.delete_rrset, {"sub_resource": True}),
        (BoundZone.change_rrset_protection, {"sub_resource": True}),
        (BoundZone.change_rrset_ttl, {"sub_resource": True}),
        (BoundZone.add_rrset_records, {"sub_resource": True}),
        (BoundZone.update_rrset_records, {"sub_resource": True}),
        (BoundZone.remove_rrset_records, {"sub_resource": True}),
        (BoundZone.set_rrset_records, {"sub_resource": True}),
    ]

    @pytest.fixture()
    def resource_client(self, client: Client):
        return client.zones

    @pytest.fixture()
    def bound_model(self, resource_client: ZonesClient, zone1):
        return BoundZone(resource_client, data=zone1)

    def test_init(self, resource_client: ZonesClient, bound_model: BoundZone):
        o = bound_model

        assert_bound_zone1(o, resource_client)

        assert o.id == 42
        assert o.name == "example1.com"
        assert o.created == isoparse("2016-01-30T23:55:00+00:00")
        assert o.mode == "primary"
        assert o.ttl == 10800
        assert o.protection == {"delete": False}
        assert o.labels == {"key": "value"}
        assert len(o.primary_nameservers) == 2

        assert isinstance(o.primary_nameservers[0], ZonePrimaryNameserver)
        assert o.primary_nameservers[0].address == "198.51.100.1"
        assert o.primary_nameservers[0].port == 53
        assert isinstance(o.primary_nameservers[1], ZonePrimaryNameserver)
        assert o.primary_nameservers[1].address == "203.0.113.1"
        assert o.primary_nameservers[1].port == 53

        assert isinstance(o.authoritative_nameservers, ZoneAuthoritativeNameservers)
        assert o.authoritative_nameservers.assigned == [
            "hydrogen.ns.hetzner.com.",
            "oxygen.ns.hetzner.com.",
            "helium.ns.hetzner.de.",
        ]
        assert o.authoritative_nameservers.delegated == [
            "hydrogen.ns.hetzner.com.",
            "oxygen.ns.hetzner.com.",
            "helium.ns.hetzner.de.",
        ]
        assert o.authoritative_nameservers.delegation_last_check == isoparse(
            "2016-01-30T23:55:00+00:00"
        )
        assert o.authoritative_nameservers.delegation_status == "valid"

        assert o.record_count == 0
        assert o.status == "ok"
        assert o.registrar == "hetzner"


class TestBoundZoneRRSet(BoundModelTestCase):
    methods = [
        BoundZoneRRSet.update_rrset,
        BoundZoneRRSet.delete_rrset,
        BoundZoneRRSet.change_rrset_protection,
        BoundZoneRRSet.change_rrset_ttl,
        BoundZoneRRSet.add_rrset_records,
        BoundZoneRRSet.update_rrset_records,
        BoundZoneRRSet.remove_rrset_records,
        BoundZoneRRSet.set_rrset_records,
    ]

    @pytest.fixture()
    def resource_client(self, client: Client):
        return client.zones

    @pytest.fixture()
    def bound_model(self, resource_client: ZonesClient, zone_rrset1):
        return BoundZoneRRSet(resource_client, data=zone_rrset1)

    def test_init(self, resource_client: ZonesClient, bound_model: BoundZoneRRSet):
        o = bound_model

        assert_bound_zone_rrset1(o, resource_client)

        assert o.id == "www/A"
        assert o.name == "www"
        assert o.type == "A"
        assert o.ttl == 3600
        assert o.labels == {"key": "value"}
        assert o.protection == {"change": False}
        assert len(o.records) == 1

        assert isinstance(o.records[0], ZoneRecord)
        assert o.records[0].value == "198.51.100.1"
        assert o.records[0].comment == "web server"

        assert isinstance(o.zone, BoundZone)
        assert o.zone.id == 42

    def test_reload(
        self,
        request_mock: mock.MagicMock,
        resource_client: ZonesClient,
        zone_rrset1,
    ):
        o = BoundZoneRRSet(
            resource_client,
            data={"id": "www/A", "zone": 42},
            complete=False,
        )
        request_mock.return_value = {"rrset": zone_rrset1}

        o.reload()

        request_mock.assert_called_with(
            method="GET",
            url="/zones/42/rrsets/www/A",
        )

        assert o.labels is not None



from __future__ import annotations

import datetime
from datetime import timezone
from unittest import mock

import pytest

from hcloud import Client
from hcloud.images import BoundImage, Image, ImagesClient
from hcloud.servers import BoundServer

from ..conftest import BoundModelTestCase


class TestBoundImage(BoundModelTestCase):
    methods = [
        BoundImage.update,
        BoundImage.delete,
        BoundImage.change_protection,
    ]

    @pytest.fixture()
    def resource_client(self, client: Client):
        return client.images

    @pytest.fixture()
    def bound_model(self, resource_client):
        return BoundImage(resource_client, data=dict(id=14))

    def test_init(self, image_response):
        bound_image = BoundImage(client=mock.MagicMock(), data=image_response["image"])

        assert bound_image.id == 4711
        assert bound_image.type == "snapshot"
        assert bound_image.status == "available"
        assert bound_image.name == "ubuntu-20.04"
        assert bound_image.description == "Ubuntu 20.04 Standard 64 bit"
        assert bound_image.image_size == 2.3
        assert bound_image.disk_size == 10
        assert bound_image.created == datetime.datetime(
            2016, 1, 30, 23, 50, tzinfo=timezone.utc
        )
        assert bound_image.os_flavor == "ubuntu"
        assert bound_image.os_version == "16.04"
        assert bound_image.architecture == "x86"
        assert bound_image.rapid_deploy is False
        assert bound_image.deprecated == datetime.datetime(
            2018, 2, 28, 0, 0, tzinfo=timezone.utc
        )

        assert isinstance(bound_image.created_from, BoundServer)
        assert bound_image.created_from.id == 1
        assert bound_image.created_from.name == "Server"
        assert bound_image.created_from.complete is False

        assert isinstance(bound_image.bound_to, BoundServer)
        assert bound_image.bound_to.id == 1
        assert bound_image.bound_to.complete is False


class TestImagesClient:
    @pytest.fixture()
    def images_client(self, client: Client):
        return ImagesClient(client)

    def test_get_by_id(
        self,
        request_mock: mock.MagicMock,
        images_client: ImagesClient,
        image_response,
    ):
        request_mock.return_value = image_response

        image = images_client.get_by_id(1)

        request_mock.assert_called_with(
            method="GET",
            url="/images/1",
        )
        assert image._client is images_client
        assert image.id == 4711
        assert image.name == "ubuntu-20.04"

    @pytest.mark.parametrize(
        "params",
        [
            {
                "name": "ubuntu-20.04",
                "type": "system",
                "sort": "id",
                "bound_to": "1",
                "label_selector": "k==v",
                "page": 1,
                "per_page": 10,
            },
            {"name": ""},
            {"include_deprecated": True},
            {},
        ],
    )
    def test_get_list(
        self,
        request_mock: mock.MagicMock,
        images_client: ImagesClient,
        two_images_response,
        params,
    ):
        request_mock.return_value = two_images_response

        result = images_client.get_list(**params)

        request_mock.assert_called_with(
            method="GET",
            url="/images",
            params=params,
        )

        images = result.images
        assert result.meta is not None

        assert len(images) == 2

        images1 = images[0]
        images2 = images[1]

        assert images1._client is images_client
        assert images1.id == 4711
        assert images1.name == "ubuntu-20.04"

        assert images2._client is images_client
        assert images2.id == 4712
        assert images2.name == "ubuntu-18.10"

    @pytest.mark.parametrize(
        "params",
        [
            {
                "name": "ubuntu-20.04",
                "type": "system",
                "sort": "id",
                "bound_to": "1",
                "label_selector": "k==v",
            },
            {"include_deprecated": True},
            {},
        ],
    )
    def test_get_all(
        self,
        request_mock: mock.MagicMock,
        images_client: ImagesClient,
        two_images_response,
        params,
    ):
        request_mock.return_value = two_images_response

        images = images_client.get_all(**params)

        params.update({"page": 1, "per_page": 50})

        request_mock.assert_called_with(
            method="GET",
            url="/images",
            params=params,
        )

        assert len(images) == 2

        images1 = images[0]
        images2 = images[1]

        assert images1._client is images_client
        assert images1.id == 4711
        assert images1.name == "ubuntu-20.04"

        assert images2._client is images_client
        assert images2.id == 4712
        assert images2.name == "ubuntu-18.10"

    def test_get_by_name(
        self,
        request_mock: mock.MagicMock,
        images_client: ImagesClient,
        one_images_response,
    ):
        request_mock.return_value = one_images_response

        with pytest.deprecated_call():
            image = images_client.get_by_name("ubuntu-20.04")

        params = {"name": "ubuntu-20.04"}

        request_mock.assert_called_with(
            method="GET",
            url="/images",
            params=params,
        )

        assert image._client is images_client
        assert image.id == 4711
        assert image.name == "ubuntu-20.04"

    def test_get_by_name_and_architecture(
        self,
        request_mock: mock.MagicMock,
        images_client: ImagesClient,
        one_images_response,
    ):
        request_mock.return_value = one_images_response

        image = images_client.get_by_name_and_architecture("ubuntu-20.04", "x86")

        params = {"name": "ubuntu-20.04", "architecture": ["x86"]}

        request_mock.assert_called_with(
            method="GET",
            url="/images",
            params=params,
        )

        assert image._client is images_client
        assert image.id == 4711
        assert image.name == "ubuntu-20.04"
        assert image.architecture == "x86"

    @pytest.mark.parametrize(
        "image", [Image(id=1), BoundImage(mock.MagicMock(), dict(id=1))]
    )
    def test_update(
        self,
        request_mock: mock.MagicMock,
        images_client: ImagesClient,
        image,
        response_update_image,
    ):
        request_mock.return_value = response_update_image

        image = images_client.update(
            image, description="My new Image description", type="snapshot", labels={}
        )

        request_mock.assert_called_with(
            method="PUT",
            url="/images/1",
            json={
                "description": "My new Image description",
                "type": "snapshot",
                "labels": {},
            },
        )

        assert image.id == 4711
        assert image.description == "My new Image description"

    @pytest.mark.parametrize(
        "image", [Image(id=1), BoundImage(mock.MagicMock(), dict(id=1))]
    )
    def test_change_protection(
        self,
        request_mock: mock.MagicMock,
        images_client: ImagesClient,
        image,
        action_response,
    ):
        request_mock.return_value = action_response

        action = images_client.change_protection(image, True)

        request_mock.assert_called_with(
            method="POST",
            url="/images/1/actions/change_protection",
            json={"delete": True},
        )

        assert action.id == 1
        assert action.progress == 0

    @pytest.mark.parametrize(
        "image", [Image(id=1), BoundImage(mock.MagicMock(), dict(id=1))]
    )
    def test_delete(
        self,
        request_mock: mock.MagicMock,
        images_client: ImagesClient,
        image,
        action_response,
    ):
        request_mock.return_value = action_response

        delete_success = images_client.delete(image)

        request_mock.assert_called_with(
            method="DELETE",
            url="/images/1",
        )

        assert delete_success is True



from __future__ import annotations

from unittest import mock

import pytest

from hcloud import Client
from hcloud.placement_groups import (
    BoundPlacementGroup,
    PlacementGroupsClient,
)

from ..conftest import BoundModelTestCase


def check_variables(placement_group: BoundPlacementGroup, expected):
    assert placement_group.id == expected["id"]
    assert placement_group.name == expected["name"]
    assert placement_group.labels == expected["labels"]
    assert placement_group.servers == expected["servers"]
    assert placement_group.type == expected["type"]


class TestBoundPlacementGroup(BoundModelTestCase):
    methods = [
        BoundPlacementGroup.update,
        BoundPlacementGroup.delete,
    ]

    @pytest.fixture()
    def resource_client(self, client: Client):
        return client.placement_groups

    @pytest.fixture()
    def bound_model(self, resource_client: PlacementGroupsClient):
        return BoundPlacementGroup(resource_client, data=dict(id=897))

    def test_init(self, placement_group_response):
        bound_placement_group = BoundPlacementGroup(
            client=mock.MagicMock(), data=placement_group_response["placement_group"]
        )

        check_variables(
            bound_placement_group, placement_group_response["placement_group"]
        )


class TestPlacementGroupsClient:
    @pytest.fixture()
    def resource_client(self, client: Client):
        return client.placement_groups

    @pytest.fixture()
    def bound_model(self, resource_client: PlacementGroupsClient):
        return BoundPlacementGroup(resource_client, data=dict(id=897))

    def test_get_by_id(
        self,
        request_mock: mock.MagicMock,
        resource_client: PlacementGroupsClient,
        placement_group_response,
    ):
        request_mock.return_value = placement_group_response

        placement_group = resource_client.get_by_id(
            placement_group_response["placement_group"]["id"]
        )

        request_mock.assert_called_with(
            method="GET",
            url="/placement_groups/897",
        )

        assert placement_group._client is resource_client

        check_variables(placement_group, placement_group_response["placement_group"])

    @pytest.mark.parametrize(
        "params",
        [
            {
                "name": "my Placement Group",
                "sort": "id",
                "label_selector": "key==value",
                "page": 1,
                "per_page": 10,
            },
            {"name": ""},
            {},
        ],
    )
    def test_get_list(
        self,
        request_mock: mock.MagicMock,
        resource_client: PlacementGroupsClient,
        two_placement_groups_response,
        params,
    ):
        request_mock.return_value = two_placement_groups_response

        result = resource_client.get_list(**params)

        request_mock.assert_called_with(
            method="GET",
            url="/placement_groups",
            params=params,
        )

        placement_groups = result.placement_groups
        assert result.meta is not None

        assert len(placement_groups) == len(
            two_placement_groups_response["placement_groups"]
        )

        for placement_group, expected in zip(
            placement_groups, two_placement_groups_response["placement_groups"]
        ):
            assert placement_group._client is resource_client

            check_variables(placement_group, expected)

    @pytest.mark.parametrize(
        "params",
        [
            {
                "name": "Corporate Intranet Protection",
                "sort": "id",
                "label_selector": "key==value",
            },
            {},
        ],
    )
    def test_get_all(
        self,
        request_mock: mock.MagicMock,
        resource_client: PlacementGroupsClient,
        two_placement_groups_response,
        params,
    ):
        request_mock.return_value = two_placement_groups_response

        placement_groups = resource_client.get_all(**params)

        params.update({"page": 1, "per_page": 50})

        request_mock.assert_called_with(
            method="GET",
            url="/placement_groups",
            params=params,
        )

        assert len(placement_groups) == len(
            two_placement_groups_response["placement_groups"]
        )

        for placement_group, expected in zip(
            placement_groups, two_placement_groups_response["placement_groups"]
        ):
            assert placement_group._client is resource_client

            check_variables(placement_group, expected)

    def test_get_by_name(
        self,
        request_mock: mock.MagicMock,
        resource_client: PlacementGroupsClient,
        one_placement_group_response,
    ):
        request_mock.return_value = one_placement_group_response

        placement_group = resource_client.get_by_name(
            one_placement_group_response["placement_groups"][0]["name"]
        )

        params = {"name": one_placement_group_response["placement_groups"][0]["name"]}

        request_mock.assert_called_with(
            method="GET",
            url="/placement_groups",
            params=params,
        )

        check_variables(
            placement_group, one_placement_group_response["placement_groups"][0]
        )

    def test_create(
        self,
        request_mock: mock.MagicMock,
        resource_client: PlacementGroupsClient,
        response_create_placement_group,
    ):
        request_mock.return_value = response_create_placement_group

        response = resource_client.create(
            name=response_create_placement_group["placement_group"]["name"],
            type=response_create_placement_group["placement_group"]["type"],
            labels=response_create_placement_group["placement_group"]["labels"],
        )

        json = {
            "name": response_create_placement_group["placement_group"]["name"],
            "labels": response_create_placement_group["placement_group"]["labels"],
            "type": response_create_placement_group["placement_group"]["type"],
        }

        request_mock.assert_called_with(
            method="POST",
            url="/placement_groups",
            json=json,
        )

        bound_placement_group = response.placement_group

        assert bound_placement_group._client is resource_client
        check_variables(
            bound_placement_group, response_create_placement_group["placement_group"]
        )

    def test_update(
        self,
        request_mock: mock.MagicMock,
        resource_client: PlacementGroupsClient,
        bound_model,
        placement_group_response,
    ):
        request_mock.return_value = placement_group_response

        placement_group = resource_client.update(
            bound_model,
            name=placement_group_response["placement_group"]["name"],
            labels=placement_group_response["placement_group"]["labels"],
        )

        request_mock.assert_called_with(
            method="PUT",
            url="/placement_groups/897",
            json={
                "labels": placement_group_response["placement_group"]["labels"],
                "name": placement_group_response["placement_group"]["name"],
            },
        )

        check_variables(placement_group, placement_group_response["placement_group"])

    def test_delete(
        self,
        request_mock: mock.MagicMock,
        resource_client: PlacementGroupsClient,
        bound_model,
    ):
        delete_success = resource_client.delete(bound_model)

        request_mock.assert_called_with(
            method="DELETE",
            url="/placement_groups/897",
        )

        assert delete_success is True



from __future__ import annotations

from unittest import mock

import pytest
from dateutil.parser import isoparse

from hcloud import Client
from hcloud.networks import (
    BoundNetwork,
    Network,
    NetworkRoute,
    NetworksClient,
    NetworkSubnet,
)
from hcloud.servers import BoundServer

from ..conftest import BoundModelTestCase


class TestBoundNetwork(BoundModelTestCase):
    methods = [
        BoundNetwork.update,
        BoundNetwork.delete,
        BoundNetwork.add_subnet,
        BoundNetwork.delete_subnet,
        BoundNetwork.add_route,
        BoundNetwork.delete_route,
        BoundNetwork.change_ip_range,
        BoundNetwork.change_protection,
    ]

    @pytest.fixture()
    def resource_client(self, client: Client):
        return client.networks

    @pytest.fixture()
    def bound_model(self, resource_client: NetworksClient):
        return BoundNetwork(resource_client, data=dict(id=14))

    def test_init(self, network_response):
        bound_network = BoundNetwork(
            client=mock.MagicMock(), data=network_response["network"]
        )

        assert bound_network.id == 1
        assert bound_network.created == isoparse("2016-01-30T23:50:11+00:00")
        assert bound_network.name == "mynet"
        assert bound_network.ip_range == "10.0.0.0/16"
        assert bound_network.protection["delete"] is False

        assert len(bound_network.servers) == 1
        assert isinstance(bound_network.servers[0], BoundServer)
        assert bound_network.servers[0].id == 42
        assert bound_network.servers[0].complete is False

        assert len(bound_network.subnets) == 2
        assert isinstance(bound_network.subnets[0], NetworkSubnet)
        assert bound_network.subnets[0].type == NetworkSubnet.TYPE_CLOUD
        assert bound_network.subnets[0].ip_range == "10.0.1.0/24"
        assert bound_network.subnets[0].network_zone == "eu-central"
        assert bound_network.subnets[0].gateway == "10.0.0.1"

        assert len(bound_network.routes) == 1
        assert isinstance(bound_network.routes[0], NetworkRoute)
        assert bound_network.routes[0].destination == "10.100.1.0/24"
        assert bound_network.routes[0].gateway == "10.0.1.1"


class TestNetworksClient:
    @pytest.fixture()
    def networks_client(self, client: Client):
        return NetworksClient(client)

    @pytest.fixture()
    def network_subnet(self):
        return NetworkSubnet(
            type=NetworkSubnet.TYPE_CLOUD,
            ip_range="10.0.1.0/24",
            network_zone="eu-central",
        )

    @pytest.fixture()
    def network_vswitch_subnet(self):
        return NetworkSubnet(
            type=NetworkSubnet.TYPE_VSWITCH,
            ip_range="10.0.1.0/24",
            network_zone="eu-central",
            vswitch_id=123,
        )

    @pytest.fixture()
    def network_route(self):
        return NetworkRoute(destination="10.100.1.0/24", gateway="10.0.1.1")

    def test_get_by_id(
        self,
        request_mock: mock.MagicMock,
        networks_client: NetworksClient,
        network_response,
    ):
        request_mock.return_value = network_response

        bound_network = networks_client.get_by_id(1)

        request_mock.assert_called_with(
            method="GET",
            url="/networks/1",
        )
        assert bound_network._client is networks_client
        assert bound_network.id == 1
        assert bound_network.name == "mynet"

    @pytest.mark.parametrize(
        "params",
        [{"label_selector": "label1", "page": 1, "per_page": 10}, {"name": ""}, {}],
    )
    def test_get_list(
        self,
        request_mock: mock.MagicMock,
        networks_client: NetworksClient,
        two_networks_response,
        params,
    ):
        request_mock.return_value = two_networks_response

        result = networks_client.get_list(**params)

        request_mock.assert_called_with(
            method="GET",
            url="/networks",
            params=params,
        )

        bound_networks = result.networks
        assert result.meta is not None

        assert len(bound_networks) == 2

        bound_network1 = bound_networks[0]
        bound_network2 = bound_networks[1]

        assert bound_network1._client is networks_client
        assert bound_network1.id == 1
        assert bound_network1.name == "mynet"

        assert bound_network2._client is networks_client
        assert bound_network2.id == 2
        assert bound_network2.name == "myanothernet"

    @pytest.mark.parametrize("params", [{"label_selector": "label1"}])
    def test_get_all(
        self,
        request_mock: mock.MagicMock,
        networks_client: NetworksClient,
        two_networks_response,
        params,
    ):
        request_mock.return_value = two_networks_response

        bound_networks = networks_client.get_all(**params)

        params.update({"page": 1, "per_page": 50})

        request_mock.assert_called_with(
            method="GET",
            url="/networks",
            params=params,
        )

        assert len(bound_networks) == 2

        bound_network1 = bound_networks[0]
        bound_network2 = bound_networks[1]

        assert bound_network1._client is networks_client
        assert bound_network1.id == 1
        assert bound_network1.name == "mynet"

        assert bound_network2._client is networks_client
        assert bound_network2.id == 2
        assert bound_network2.name == "myanothernet"

    def test_get_by_name(
        self,
        request_mock: mock.MagicMock,
        networks_client: NetworksClient,
        one_network_response,
    ):
        request_mock.return_value = one_network_response

        bound_network = networks_client.get_by_name("mynet")

        params = {"name": "mynet"}

        request_mock.assert_called_with(
            method="GET",
            url="/networks",
            params=params,
        )

        assert bound_network._client is networks_client
        assert bound_network.id == 1
        assert bound_network.name == "mynet"

    def test_create(
        self,
        request_mock: mock.MagicMock,
        networks_client: NetworksClient,
        network_create_response,
    ):
        request_mock.return_value = network_create_response

        networks_client.create(name="mynet", ip_range="10.0.0.0/8")

        request_mock.assert_called_with(
            method="POST",
            url="/networks",
            json={"name": "mynet", "ip_range": "10.0.0.0/8"},
        )

    def test_create_with_expose_routes_to_vswitch(
        self,
        request_mock: mock.MagicMock,
        networks_client: NetworksClient,
        network_create_response_with_expose_routes_to_vswitch,
    ):
        request_mock.return_value = (
            network_create_response_with_expose_routes_to_vswitch
        )
        networks_client.create(
            name="mynet", ip_range="10.0.0.0/8", expose_routes_to_vswitch=True
        )

        request_mock.assert_called_with(
            method="POST",
            url="/networks",
            json={
                "name": "mynet",
                "ip_range": "10.0.0.0/8",
                "expose_routes_to_vswitch": True,
            },
        )

    def test_create_with_subnet(
        self,
        request_mock: mock.MagicMock,
        networks_client: NetworksClient,
        network_subnet,
        network_create_response,
    ):
        request_mock.return_value = network_create_response

        networks_client.create(
            name="mynet", ip_range="10.0.0.0/8", subnets=[network_subnet]
        )

        request_mock.assert_called_with(
            method="POST",
            url="/networks",
            json={
                "name": "mynet",
                "ip_range": "10.0.0.0/8",
                "subnets": [
                    {
                        "type": NetworkSubnet.TYPE_CLOUD,
                        "ip_range": "10.0.1.0/24",
                        "network_zone": "eu-central",
                    }
                ],
            },
        )

    def test_create_with_subnet_vswitch(
        self,
        request_mock: mock.MagicMock,
        networks_client: NetworksClient,
        network_subnet,
        network_create_response,
    ):
        request_mock.return_value = network_create_response

        network_subnet.type = NetworkSubnet.TYPE_VSWITCH
        network_subnet.vswitch_id = 1000
        networks_client.create(
            name="mynet", ip_range="10.0.0.0/8", subnets=[network_subnet]
        )

        request_mock.assert_called_with(
            method="POST",
            url="/networks",
            json={
                "name": "mynet",
                "ip_range": "10.0.0.0/8",
                "subnets": [
                    {
                        "type": NetworkSubnet.TYPE_VSWITCH,
                        "ip_range": "10.0.1.0/24",
                        "network_zone": "eu-central",
                        "vswitch_id": 1000,
                    }
                ],
            },
        )

    def test_create_with_route(
        self,
        request_mock: mock.MagicMock,
        networks_client: NetworksClient,
        network_route,
        network_create_response,
    ):
        request_mock.return_value = network_create_response

        networks_client.create(
            name="mynet", ip_range="10.0.0.0/8", routes=[network_route]
        )

        request_mock.assert_called_with(
            method="POST",
            url="/networks",
            json={
                "name": "mynet",
                "ip_range": "10.0.0.0/8",
                "routes": [{"destination": "10.100.1.0/24", "gateway": "10.0.1.1"}],
            },
        )

    def test_create_with_route_and_expose_routes_to_vswitch(
        self,
        request_mock: mock.MagicMock,
        networks_client,
        network_route,
        network_create_response_with_expose_routes_to_vswitch,
    ):
        request_mock.return_value = (
            network_create_response_with_expose_routes_to_vswitch
        )
        networks_client.create(
            name="mynet",
            ip_range="10.0.0.0/8",
            routes=[network_route],
            expose_routes_to_vswitch=True,
        )

        request_mock.assert_called_with(
            method="POST",
            url="/networks",
            json={
                "name": "mynet",
                "ip_range": "10.0.0.0/8",
                "routes": [{"destination": "10.100.1.0/24", "gateway": "10.0.1.1"}],
                "expose_routes_to_vswitch": True,
            },
        )

    def test_create_with_route_and_subnet(
        self,
        request_mock: mock.MagicMock,
        networks_client: NetworksClient,
        network_subnet,
        network_route,
        network_create_response,
    ):
        request_mock.return_value = network_create_response

        networks_client.create(
            name="mynet",
            ip_range="10.0.0.0/8",
            subnets=[network_subnet],
            routes=[network_route],
        )

        request_mock.assert_called_with(
            method="POST",
            url="/networks",
            json={
                "name": "mynet",
                "ip_range": "10.0.0.0/8",
                "subnets": [
                    {
                        "type": NetworkSubnet.TYPE_CLOUD,
                        "ip_range": "10.0.1.0/24",
                        "network_zone": "eu-central",
                    }
                ],
                "routes": [{"destination": "10.100.1.0/24", "gateway": "10.0.1.1"}],
            },
        )

    @pytest.mark.parametrize(
        "network", [Network(id=1), BoundNetwork(mock.MagicMock(), dict(id=1))]
    )
    def test_update(
        self,
        request_mock: mock.MagicMock,
        networks_client: NetworksClient,
        network,
        response_update_network,
    ):
        request_mock.return_value = response_update_network

        network = networks_client.update(
            network, name="new-name", expose_routes_to_vswitch=True
        )

        request_mock.assert_called_with(
            method="PUT",
            url="/networks/1",
            json={"name": "new-name", "expose_routes_to_vswitch": True},
        )

        assert network.id == 4711
        assert network.name == "new-name"
        assert network.expose_routes_to_vswitch is True

    @pytest.mark.parametrize(
        "network", [Network(id=1), BoundNetwork(mock.MagicMock(), dict(id=1))]
    )
    def test_change_protection(
        self,
        request_mock: mock.MagicMock,
        networks_client: NetworksClient,
        network,
        action_response,
    ):
        request_mock.return_value = action_response

        action = networks_client.change_protection(network, True)

        request_mock.assert_called_with(
            method="POST",
            url="/networks/1/actions/change_protection",
            json={"delete": True},
        )

        assert action.id == 1
        assert action.progress == 0

    @pytest.mark.parametrize(
        "network", [Network(id=1), BoundNetwork(mock.MagicMock(), dict(id=1))]
    )
    def test_delete(
        self,
        request_mock: mock.MagicMock,
        networks_client: NetworksClient,
        network,
        action_response,
    ):
        request_mock.return_value = action_response

        delete_success = networks_client.delete(network)

        request_mock.assert_called_with(
            method="DELETE",
            url="/networks/1",
        )

        assert delete_success is True

    @pytest.mark.parametrize(
        "network", [Network(id=1), BoundNetwork(mock.MagicMock(), dict(id=1))]
    )
    def test_add_subnet(
        self,
        request_mock: mock.MagicMock,
        networks_client: NetworksClient,
        network,
        action_response,
        network_subnet,
    ):
        request_mock.return_value = action_response

        action = networks_client.add_subnet(network, network_subnet)

        request_mock.assert_called_with(
            method="POST",
            url="/networks/1/actions/add_subnet",
            json={
                "type": NetworkSubnet.TYPE_CLOUD,
                "ip_range": "10.0.1.0/24",
                "network_zone": "eu-central",
            },
        )

        assert action.id == 1
        assert action.progress == 0

    @pytest.mark.parametrize(
        "network", [Network(id=1), BoundNetwork(mock.MagicMock(), dict(id=1))]
    )
    def test_add_subnet_vswitch(
        self,
        request_mock: mock.MagicMock,
        networks_client: NetworksClient,
        network,
        action_response,
        network_vswitch_subnet,
    ):
        request_mock.return_value = action_response

        action = networks_client.add_subnet(network, network_vswitch_subnet)

        request_mock.assert_called_with(
            method="POST",
            url="/networks/1/actions/add_subnet",
            json={
                "type": NetworkSubnet.TYPE_VSWITCH,
                "ip_range": "10.0.1.0/24",
                "network_zone": "eu-central",
                "vswitch_id": 123,
            },
        )

        assert action.id == 1
        assert action.progress == 0

    @pytest.mark.parametrize(
        "network", [Network(id=1), BoundNetwork(mock.MagicMock(), dict(id=1))]
    )
    def test_delete_subnet(
        self,
        request_mock: mock.MagicMock,
        networks_client: NetworksClient,
        network,
        action_response,
        network_subnet,
    ):
        request_mock.return_value = action_response

        action = networks_client.delete_subnet(network, network_subnet)

        request_mock.assert_called_with(
            method="POST",
            url="/networks/1/actions/delete_subnet",
            json={"ip_range": "10.0.1.0/24"},
        )

        assert action.id == 1
        assert action.progress == 0

    @pytest.mark.parametrize(
        "network", [Network(id=1), BoundNetwork(mock.MagicMock(), dict(id=1))]
    )
    def test_add_route(
        self,
        request_mock: mock.MagicMock,
        networks_client: NetworksClient,
        network,
        action_response,
        network_route,
    ):
        request_mock.return_value = action_response

        action = networks_client.add_route(network, network_route)

        request_mock.assert_called_with(
            method="POST",
            url="/networks/1/actions/add_route",
            json={"destination": "10.100.1.0/24", "gateway": "10.0.1.1"},
        )

        assert action.id == 1
        assert action.progress == 0

    @pytest.mark.parametrize(
        "network", [Network(id=1), BoundNetwork(mock.MagicMock(), dict(id=1))]
    )
    def test_delete_route(
        self,
        request_mock: mock.MagicMock,
        networks_client: NetworksClient,
        network,
        action_response,
        network_route,
    ):
        request_mock.return_value = action_response

        action = networks_client.delete_route(network, network_route)

        request_mock.assert_called_with(
            method="POST",
            url="/networks/1/actions/delete_route",
            json={"destination": "10.100.1.0/24", "gateway": "10.0.1.1"},
        )

        assert action.id == 1
        assert action.progress == 0

    @pytest.mark.parametrize(
        "network", [Network(id=1), BoundNetwork(mock.MagicMock(), dict(id=1))]
    )
    def test_change_ip_range(
        self,
        request_mock: mock.MagicMock,
        networks_client: NetworksClient,
        network,
        action_response,
    ):
        request_mock.return_value = action_response

        action = networks_client.change_ip_range(network, "10.0.0.0/12")

        request_mock.assert_called_with(
            method="POST",
            url="/networks/1/actions/change_ip_range",
            json={"ip_range": "10.0.0.0/12"},
        )

        assert action.id == 1
        assert action.progress == 0



from __future__ import annotations

import pytest

from hcloud.deprecation import DeprecationInfo


@pytest.mark.parametrize(
    "value",
    [
        (DeprecationInfo(),),
    ],
)
def test_eq(value):
    assert value.__eq__(value)
